@@ -15,6 +15,10 @@ pub use node::{Node, NodeGetHelper, NodeRebuildHelper};
 mod iter;
 pub use iter::{ConsumingIter, Iter, IterMut};
 
+mod error;
+#[cfg(feature = "high_assurance")]
+pub use error::SGErr;
+
 #[allow(clippy::module_inception)]
 mod tree;
 pub use tree::SGTree;
